$SAVE
C$NOFLOATCALLS
C$NODEBUG
************************************************************
C --- PROGRAM  R A S C A L
C
C     PROGRAM RASCAL (RESPONSE SPECTRUM AND ACCELEROGRAM SCALING)
C     COMPUTES BRUNE FOURIER AMPLITUDE BASED ON THE METHOD SUGGESTED
C     BY J. N. BRUNE (BSSA, VOL. 75, SEPTEMBER 1970) AND COMPUTES PEAK
C     ACCELERATION, PEAK VELOCITY, RESPONSE SPECTRAL ACCELERATION AND
C     RESPONSE SPECTRAL VELOCITY BASED ON THE METHOD SUGGESTED BY D. M.
C     BOORE (BSSA, VOL. 73, DECEMBER 1983) BY USING RANDOM VIBRATION
C     THEORY (RVT) TECHNIQUES.
C
C     THE PROGRAM ALSO GENERATES SYNTHETIC TIME HISTORY (ACCELERATION,
C     VELOCITY OR DISPLACEMENT) BY COMPUTING THE FFT AND EXTRACTING THE
C     PHASE OF AN INPUT ACCELEROGRAM (INTERNAL OR EXTRENAL BASE) AND
C     COMBINES THE COMPUTED BRUNE FOURIER AMPLITUDE TO GENERATE THE
C     OUTPUT TIME HISTORIES.
C
C     THIS PROGRAM HAS THE CAPABILITY OF COMPUTING THE HALF-SPACE (ROCK
C     OUTCROP) OR SITE (SINGLE-LAYERED SOIL) RESPONSE AND OF GENERATING
C     OUTPUT VELOCITY OR DISPLACEMENT TIME HISTORY AS WELL.
C
C --- CODED BY KIN W. LEE, 1985.; VER 1.0
C --- CONVERTED TO MS-DOS SYSTEM BY Shyh-Shiun Lai, December 1986; VER.1.1
C --- MAGNITUDE SELECTION LOGIC CHANGED (Sept. 1987, Walt Silva); VER. 1.2
C     WOODWARD-CLYDE CONSULTANTS
C     ADDED INCLUDE FILE RASCAL.H(APRIL 2000, WALT SILVA): VER. 1.3
C     INCLUDED ADDITIONAL INTERNAL TIME HISTORIES (JANUARY, 2006): VER. 1.4
C
C ************************************************************************
C
C --- U S E R ' S   G U I D E
C
C     *** ITEMS 1 - 6 ARE REQUIRED IN THE INPUT DATA FILE
C
C --- BASIC PARAMETERS :
C
C     1.  TITLE : FORMAT(A80)
C
C         TITLE IS THE TITLE OF THE RUN.
C
C     2.  OFILE : FORMAT(A80)
C
C         OFILE IS THE OUTPUT FILE NAME OF THE RUN.
C               OUTPUT FILES OF SPECTRAL VALUES, ACCELERATION, VELOCITY,
C               AND DISPLACEMENT TIME HISTORIES ARE AUTOMATICALLY CREATED
C               SEPARATELY UNDER THIS OFILE NAME WITH SUFFIXES OF R, S,
C               A, V, AND D RESPECTIVELY, AND OF THE LAST ITERATION NUMBER
C               AT THE END OF EACH FILE.
C
C     3.  SDROP, DENS, D, H, SV : FORMAT(FREE)
C
C         SDROP IS THE STRESS DROP (BARS).
C         DENS IS THE MEDIUM DENSITY (GM/CC).
C         D IS THE EPICENTRAL DISTANCE (KM).
C         H IS THE SOURCE DEPTH (KM).
C         SV IS THE SHEAR WAVE VELOCITY OF THE HALF-SPACE (KM/SEC).
C
C     4.  Q, FQ, ALPHA, MW : FORMAT(FREE)
C
C         Q IS THE FREQUENCY DEPENDENT QUALITY FACTOR OF THE HALF-SPACE
C           FOR Q FILTER, WHERE Q(F) = Q * ( F / FQ ) ** ALPHA.
C         FQ IS THE CONTROL FREQUENCY FOR THE Q QUALITY FACTOR (HERTZ).
C         ALPHA IS THE EXPONENTIAL CONSTANT FOR THE Q OPERATOR.
C         MW IS THE MOMENT MAGNITUDE.
C
C     5.  DAMP, FMAX, N, CAP : FORMAT(FREE)
C
C         DAMP IS THE SPECTRAL DAMPING (PERCENT).
C              (MUST BE BETWEEN 1 TO 10 %).
C         FMAX IS THE BUTTERWORTH CORNER FREQUENCY (HERTZ).
C              (IF FMAX = 0, NO FILTERING IS PERFORMED).
C         N IS THE FMAX ORDER NUMBER.
C           (IF N = 0, NO FILTERING IS PERFORMED).
C         CAP IS THE CONSTANT, KAPPA FOR NEAR-SITE EXPONENTIAL
C             FILTERING.
C             (IF CAP = 0.0, NO FILTERING IS PERFORMED).
C
C --- FUNCTIONAL KEYS :
C
C     6.  KEY1, KEY2, KEY3, KEY4, KEY5, KEY6 : FORMAT(FREE)
C
C         KEY1 IS THE KEY FOR COMPUTING SPECIFIED RESPONSE.
C              IF KEY1 = 0 : COMPUTES HALF-SPACE RESPONSE.
C              IF KEY1 = 1 : COMPUTES SITE RESPONSE.
C         KEY2 IS THE KEY FOR APPLYING NEAR-SITE AMPLIFICATION FACTOR
C              AS A FUNCTION OF FREQUENCY.
C              IF KEY2 = 0 : NO AMPLIFICATION FACTORS ARE APPLIED.
C              IF KEY2 = 1 : AMPLIFICATION FACTORS ARE APPLIED.
C         KEY3 IS THE KEY FOR CHANGING GEOMETRICAL ATTENUATION FOR
C              R > 100 KM.
C              IF KEY3 = 0 : BODY WAVE ( 1 / R ) IS RETAINED FOR
C                            FOR R > 100 KM.
C              IF KEY3 = 1 : ATTENUATION FOR R > 100 KM. IS APPLIED.
C                            ( FOR R < 100 KM., ATTENUATION = 1 / R ).
C                            ( FOR R > 100 KM., ATTENUATION = 1 / 100 *
C                              SQRT( 100 / R ) ).
C         KEY4 IS THE KEY FOR INCREASING EFFECTIVE SOURCE DURATION DUE
C              TO SURFACE WAVE CONTRIBUTION.
C              IF KEY4 = 0 : DURATION = 1 / FC .
C              IF KEY4 = 1 : DURATION = 1 / FC + 0.05 * R .
C                 (FROM R. B. HERRMANN, BSSA, VOL. 75, OCTOBER 1985).
C         KEY5 IS THE KEY FOR GENERATING SYNTHETIC ACCELERATION TIME
C              HISTORY.
C              IF KEY5 = 0 : NO ARIFICIAL ACCELERATION TIME HISTORY IS
C                            GENERATED. JUST BASIC RESPONSE IS COMPUTED.
C                            IN THIS CASE, NO NUMBER OF ITERATIONS AND
C                            DESIGN (TARGET) RESPONSE SPECTRUM ARE
C                            REQUIRED. SET KEY6 = 0 !!!
C              IF KEY5 > 0 : SYNTHETIC ACCELERATION TIME HISTORY IS
C                            GENERATED USING PHASES OF THE BUILT-IN
C                            ACCELERATION TIME HISTORY.
C              IF KEY5 < 0 : SYNTHETIC ACCELERATION TIME HISTORY IS
C                            GENERATED USING PHASES OF THE INPUT
C                            (USER-SUPPLIED) ACCELERATION TIME HISTORY.
C         KEY6 IS THE KEY FOR WRITING OUTPUT TIME HISTORIES, OFILE.
C              IF KEY6 = 0 : ONLY OUTPUT ACCELERATION TIME HISTORY OF THE
C                            LAST ITERATION IS WRITTEN IN OFILE.A, OR NO
C                            OUTPUT TIME HISTORIES ARE WRITTEN AT ALL IF
C                            KEY5 = 0.
C              IF KEY6 = 1 : ONLY OUTPUT ACCELERATION AND VELOCITY TIME
C                            HISTORIES ARE WRITTEN IN OFILE.A AND
C                            OFILE.V, RESPECTIVELY.
C              IF KEY6 = 2 : ONLY OUTPUT ACCELERATION AND DISPLACEMENT
C                            TIME HISTORIES ARE WRITTEN IN OFILE.A AND
C                            OFILE.D, RESPECTIVELY.
C              IF KEY6 = 3 : ALL OUTPUT ACCELERATION, VELOCITY AND
C                            DISPLACEMENT TIME HISTORIES ARE WRITTEN IN
C                            OFILE.A, OFILE.V AND OFILE.D, RESPECTIVELY.
C
C --- PARAMETERS FOR THE FUNCTION KEYS :
C
C --- SOIL RESPONSE : SKIP THIS GROUP 7 IF KEY1 = 0 !!!
C
C    7.1  DEN1, SV1, DAMP1, THICK, RD : FORMAT(FREE)
C
C         DEN1 IS THE SOIL DENSITY (GM/CC).
C         SV1 IS THE SHEAR WAVE VELOCITY OF THE SOIL (M/SEC).
C         DAMP1 IS THE DAMPING RATIO OF THE SOIL (PERCENT).
C         THICK IS THE THICKNESS OF THE SOIL LAYER (M).
C         RD IS THE RECEIVER DEPTH (DEPTH OF INTEREST) (M).
C            (RD MUST BE .LE. THICK).
C
C    7.2  DEN2, SV2 : FORMAT(FREE)
C
C         DEN2 IS THE BEDROCK DENSITY (GM/CC).
C         SV2 IS THE SHEAR WAVE VELOCITY OF THE BEDROCK (M/SEC).
C
C         *** NOTE : DENSITY UNITS IN DEN1 AND DEN2 CAN BE IN OTHER
C                    UNITS IF CONSISTENT !!!
C                    LENGTH UNITS IN THICK, RD, SV1 AND SV2 CAN BE IN
C                    OTHER UNITS IF CONSISTENT !!!
C
C --- NEAR-SITE AMPLIFICATION FACTORS : SKIP THIS GROUP 8 IF KEY1 = 2
C         OR IF KEY2 = 0 !!!
C
C     8.  NAMP, AFREQ(I), AMFAC(I) : FORMAT(FREE)
C
C         NAMP IS THE NUMBER OF AMPLIFICATION FACTORS.
C         AFREQ(I) IS THE FREQUENCY AT WHICH AMPLIFICATION IS APPLIED
C                  (HERTZ).
C         AMFAC(I) IS THE AMPLIFICATION FACTOR AT AFREQ(I).
C
C         *** REPEAT (AFREQ(I),AMFAC(I)) PAIR NAMP TIMES. (MAXIMUM 100) !!!
C
C         *** NOTE : FOR FREQUENCY < AFREQ(1), AMFAC(1) IS USED.
C                    FOR FREQUENCY > AFREQ(NAMP), AMFAC(NAMP) IS USED.
C
C --- INPUT ACCELERATION TIME HISTORY : SKIP THIS GROUP 9 IF KEY5 .LE. 0 !!!
C
C    9.1  AFILE : FORMAT(A80)
C
C         AFILE IS THE INPUT ACCELERATION TIME HISTORY FILE NAME.
C
C    9.2  NPT, NHEAD, DT, FMT : FORMAT(2I5,F10.4,A40)
C
C         NPT IS THE NUMBER OF POINTS IN THE INPUT ACCELERATION TIME
C             HISTORY. (MAXIMUM 65536).
C         NHEAD IS THE NUMBER OF HEADER CARDS IN THE INPUT ACCELERATION
C               TIME HISTORY.
C         DT IS THE TIME INCREMENT OF THE INPUT ACCELERATION TIME
C            HISTORY (SECONDS).
C         FMT IS THE READ FORMAT OF THE INPUT ACCELERATION TIME HISTORY.
C             DEFAULT IS (8F9.6).
C
C --- OUTPUT TIME HISTORIES : SKIP THIS GROUP 10 IF KEY5 = 0 !!!
C
C   10.1  FACTA, FACTV, FACTD : FORMAT(FREE)
C
C         FACTA, FACTV & FACTD ARE THE NORMALIZING FACTORS FOR THE
C                OUTPUT ACCELERATION (G'S), VELOCITY (CM/SEC) AND
C                DISPLACEMENT (CM) TIME HISTORIES, RESPECTIVELY.
C         IF FACTA,V,D = 0.0 : NO NORMALIZATION IS APPLIED.
C         IF FACTA,V,D < 0.0 : THE OUTPUT TIME HISTORIES ARE NORMALIZED
C                              TO THE COMPUTED PEAK VALUES BASED ON THE
C                              RANDOM VIBRATION THEORY, RESPECTIVELY.
C         IF FACTA,V,D > 0.0 : THE OUTPUT TIME HISTORIES ARE NORMALIZED
C                              TO FACTA,V,D, RESPECTIVELY.
C
C   10.2  NIT1, NIT2 : FORMAT(FREE)
C
C         NIT1 IS THE NUMBER OF ITERATIONS USED FOR RVT SPECTRAL MATCHING.
C         NIT2 IS THE NUMBER OF ITERATIONS USED FOR SDF SPECTRAL MATCHING.
C
C         *** NOTE : 2 TO 3 ITERATIONS FOR EACH CASE IS RECOMMENDED.
C
C --- DESIGN (TARGET) RESPONSE SPECTRUM : SKIP THIS GROUP 10.3
C                                         IF NIT1 AND NT2 = 0 !!!
C
C   10.3  NPSV, TFREQ(I), TSV(I) : FORMAT(FREE)
C
C         NPSV IS THE NUMBER OF DESIGN (TARGET) SPECTRAL VALUES.
C              (MAXIMUM 100).
C              IF NPSV > 0 : SPECTRAL VALUES (TSV(I)) MUST BE IN
C                            ACCELERATION (G'S).
C              IF NPSV < 0 : SPECTRAL VALUES (TSV(I)) MUST BE IN
C                            VELOCITY (CM/SEC).
C         TFREQ(I) IS THE DESIGN (TARGET) FREQUENCY (HERTZ).
C         TSV(I) IS THE DESIGN (TARGET) SPECTRAL VALUES AT TFREQ(I).
C
C         *** REPEAT (TFREQ(I),TSV(I)) PAIR ABS(NPSV) TIMES !!!
C
C         *** NOTE:    SET THE DESIGN FREQUENCY RANGE BEYOND 0.1 TO 34.0
C                      HERTZ, IF POSSIBLE. I.E. SET TFREQ(1) .LE. 0.1 HZ
C                      AND TFREQ(NPSV) .GE. 34.0 HERTZ. THIS RANGE IS SET
C                      UP FOR INTERPOLATING THE DESIGN SPECTRAL VALUES AT
C                      THE OSCILLATOR FREQUENCY REGIME !!!
C
C --- FILTER PARAMETERS
C
C   10.4  FC1, NF1, FC2, NF2 : FORMAT(FREE)
C
C         FC1 IS THE CORNER FREQUENCY OF THE FIRST BUTTERWORTH FILTER,
C             (-VE FOR REMOVAL). (HERTZ).
C         NF1 IS THE ORDER NUMBER (NO. OF POLES) OF THE FIRST
C             BUTTERWORTH FILTER. (-VE FOR HIGH PASS).
C         FC2 IS THE CORNER FREQUENCY OF THE SECOND BUTTERWORTH FILTER,
C             (-VE FOR REMOVAL). (HERTZ).
C         NF2 IS THE ORDER NUMBER (NO. OF POLES) OF THE SECOND
C             BUTTERWORTH FILTER. (-VE FOR HIGH PASS).
C
C             NF1 = 0. NO FIRST FILTER
C             NF2 = 0. NO SECOND FILTER
C
C         *** NOTE : FILTERS ARE APPLIED TO SOURCE SPECTRUM.
C
C ........................................................................
C
C --- FREQUENCY DATA FILE FOR RESPONSE SPECTRUM
C
C         FREQ(I) : FORMAT (F10.7)
C
C         FREQUENCY POINTS AT WHICH RESPONSE SPECTRUM IS COMPUTED.
C         VALUES SHOULD BE IN INCREASING ORDER. MAXIMUM NUMBER OF
C         POINTS IS 145.
C
C         *** NOTE: 1. NYQUIST FRREQUENCY OF INPUT TIME HISTORY MUST
C                      BE .GE. HIGHEST RESPONSE SPECTRUM FREQUENCY.
C                      AS A GUIDE, THE SDF RESPONSE SPECTRUM SHOULD
C                      NOT BE COMPUTED FOR FREQUENCIES EXCEEDING
C                      (0.7 * NYQUIST).
C
C                   2. THE FIRST TWO FREQUENCIES ARE DUMMY FREQUENCIES
C                      USED FOR COMPUTING PEAK ACCELERATION AND PEAK
C                      VELOCITY, RESPECTIVELY.
C
C ........................................................................
C --- PROGRAM I/O
C
C     AT THE START OF THE PROGRAM, THE NAMES OF THREE FILES ARE REQUESTED.
C     THESE ARE THE :
C
C           1. INPUT DATA FILE - REQUIRED AND OPTIONAL RUN PARAMETERS
C                                (UNIT 5).
C           2. OUTPUT DATA FILE - CONTAINS ECHO OF INPUT PARAMETERS, PEAK
C                                 VALUES, AND OTHER DATA (UNIT 6).
C           3. FREQUENCY DATA FILE - LIST OF OSCILLATOR FREQUENCY POINTS
C                                    (UNIT 7).
C
C     IF AN ACCELERATION TIME HISTORY IS TO BE GENERATED, THE TIME HISTORY
C     FROM WHICH THE PHASE IS EXTRACTED IS READ FROM UNIT 5. THE PHASE
C     MAY BE GENERATED FROM A USER SUPPLIED TIME HISTORY OR THE BUILT-IN
C     TIME HISTORIES PROVIDED.
C
C     THE OUTPUT SPECTRAL VALUES AND TIME HISTORIES (IF REQUESTED) ARE
C     WRITTEN AS DESCRIBED IN ITEM 2 TO :
C
C           1. RVT SPECTRAL VALUES - UNIT 8
C           2. SDF SPECTRAL VALUES - UNIT 9
C           3. OUTPUT ACCELERATION TIME HISTORY - UNIT 11
C           4. OUTPUT VELOCITY TIME HISTORY - UNIT 12
C           5. OUTPUT DISPLACEMENT TIME HISTORY - UNIT 13
C
C --- E N D   O F  P R O G R A M
C
***************************************************************
      INCLUDE 'RASCAL.H'
      CHARACTER TITLE*80,OFILE*80,AFILE*80,WORD*80,
     &          FILE8*80,FILE9*80,FILE11*80,FILE12*80,FILE13*80,
     +          SUF8*2,SUF9*2,SUF11*2,SUF12*2,SUF13*2,
     +          FMAT*80,FMT*40
      CHARACTER*1 CIT1(2),CIT(2),OFILE1(80),FLAG(MAXP)
      CHARACTER*14 FIN,FOUT,FIN1
*
      REAL  MW
C$LARGE: FREQ,FAS,RSA,RSV,PAA,PRV,TSVI,SPRAT,TFREQ,TSV,FLAG
      DIMENSION  FREQ(MAXP),FAS(MAXP), RSA(MAXP), RSV(MAXP), PAA(MAXP),
     &           PRV(MAXP), TSVI(MAXP), SPRAT(MAXP), TFREQ(MAXTAR), TSV(MAXTAR)
*
      COMPLEX  RE, HS
      DOUBLE PRECISION FUNX0,FUNX2,FUNX4,SM0,SM2,SM4
      COMMON  /FNX/ FUNX0(MAXB), FUNX2(MAXB), FUNX4(MAXB)
      COMMON     SFREQ(MAXB), SFAS(MAXB), SWI(MAXB), SPRATS(MAXB),
     &           CP(MAXF), SP(MAXF), OTH(MAXT)
      COMMON  /PIDATA/ PI, PI2
      COMMON  /FADATA/ KEY1, KEY2, C, FC, PIR, Q, FQ, ALPHA, FMAX, N2,
     &                 CAP, PICAP
      COMMON  /AMDATA/ AFREQ(MAXF), AMFAC(MAXF), NAMP
      COMMON  /SMDATA/ SV1, DAMP1, THICK, ARATIO, RD, RK, CK
      COMMON  /TMDATA/ NPIA, NPT, NPT2, NP2, NPFR, DT,
     &                 A(MAXT), PFREQ(MAXF), PFAS(MAXF)
      COMMON /SPDATA/ DAMP, NFREQ, WI(MAXP), PER(MAXP)
C
C --- OSCILLATOR (WCC) FREQUENCY REGIME
C
C ------ NOTE : THE FIRST TWO FREQUENCIES ARE DUMMY FREQUENCIES USED
C               FOR COMPUTING PEAK ACCELERATION AND PEAK VELOCITY,
C               RESPECTIVELY.
C
      WRITE(*,9999)
9999  FORMAT(//,1X,'INPUT  DATA FILE NAME = ',\)
      READ(*,'(A)') FIN
      WRITE(*,9998)
9998  FORMAT(//,1X,'OUTPUT DATA FILE NAME = ',\)
      READ(*,'(A)') FOUT
*
      WRITE(*,9997)
9997  FORMAT(//,1X,'INPUT FREQUENCY DATA FILE NAME = ',\)
      READ(*,'(A)') FIN1
*
      OPEN(5,FILE=FIN,STATUS='OLD')
      OPEN(6,FILE=FOUT,STATUS='NEW')
      OPEN(7,FILE=FIN1,STATUS='OLD')
C
C --- READ FREQUENCY DATA FILE
C
      I = 1
9995  READ (7, '(F10.3)', END = 9996) FREQ(I)
      I = I+1
      GO TO 9995
9996  CONTINUE
      CLOSE (7)
      NFREQ = I - 1
      READ(5,1100) TITLE
      READ(5,1100) OFILE
      READ(5,*) SDROP, DENS, D, H, SV
      READ(5,*) Q, FQ, ALPHA, MW
      READ(5,*) DAMP, FMAX, N, CAP
      READ(5,*) KEY1, KEY2, KEY3, KEY4, KEY5, KEY6
C
      WRITE (6, 1200) TITLE
      WRITE (6, 1300) OFILE
      WRITE (6, 1340) SDROP, DENS, D, H, SV, Q, FQ, ALPHA, MW, DAMP,
     &            FMAX, N, CAP
      WRITE (6, 1380) KEY1, KEY2, KEY3, KEY4, KEY5, KEY6
C
C --- FUNCTIONAL KEY1 OPERATION
C
      WRITE (6, 1200) TITLE
      WRITE (6, 1400)
C
      IF ( KEY1 .EQ. 0 ) THEN
         WRITE (6, 1410)
      ELSE
         READ(5,*) DEN1, SV1, DAMP1, THICK, RD
         READ(5,*) DEN2, SV2
C
         ARATIO = ( DEN1 * SV1 ) / ( DEN2 * SV2 )
         WRITE (6, 1420)
         WRITE (6, 1430) DEN1, SV1, DAMP1, THICK, RD, DEN2, SV2
         DAMP1 = DAMP1 / 100.0
      END IF
C
C --- FUNCTIONAL KEY2 OPERATION
C
      IF ( KEY2 .EQ. 0 ) THEN
         WRITE (6, 1440)
      ELSE
         READ(5,*) NAMP, ( AFREQ(I), AMFAC(I), I=1,NAMP )
         WRITE (6, 1460)
         DO 120, I=1,NAMP
  120    WRITE (6, 1480) I, AFREQ(I), AMFAC(I)
      END IF
C
C --- INITIALIZES CONSTANT PARAMETERS
C
C     NFREQ = MAXP
      AMPF = 1.0
      DAMP = DAMP / 100.0
      DAMP2 = DAMP * 2.0
      N2 = N * 2
      PI = 3.141592653589793
      PI2 = 2.0 * PI
      PICAP = PI * CAP
      SRHPI = SQRT( 0.5 * PI )
      EULER = 0.5772156649
C
      SDAMP = 100.0 / ( 2.0 * Q )
      FC = 21.07 * SV * ( SDROP ** ( 1.0 / 3.0 ) ) * EXP(-1.15 * MW)
      SR = ( 2.34 * SV ) / ( PI2 * FC )
      R = SQRT( D * D + H * H )
C
C --- FUNCTIONAL KEY3 OPERATION
C
      ATTEN = 1.0 / R
      IF ( R .GT. 100.0 .AND. KEY3 .EQ. 1 ) THEN
         ATTEN = 0.1 / SQRT(R)
         WRITE (6, 1500) ATTEN
      ELSE
         WRITE (6, 1520) ATTEN
      END IF
C
      C = ( 7.8 * SDROP * SR * ATTEN ) / ( DENS * SV )
C
C --- FUNCTIONAL KEY4 OPERATION
C
      HERR = 0.0
      IF ( KEY4 .EQ. 0 ) THEN
         WRITE (6, 1540)
      ELSE
         HERR = 0.05 * R
         WRITE (6, 1560) HERR
      END IF
C
      DRVT = 1.0 / FC + HERR
      WRITE (6, 1580) DRVT
C
C --- FUNCTIONAL KEY5 OPERATION
C
      NP2 = 12
      NPT2 = 4098
      NPT = 4096
      NHEAD = 3
      DT = 0.01
      FMT = '(8F9.6)'
C
      NPFR = 2049
      PFR1 = 0.0244141
      DPFR = PFR1
C
      FACTA = 0.0
      FACTV = 0.0
      FACTD = 0.0
C
      TSVMX = 0.0
      DO 130 I=1,NFREQ
  130 TSVI(I) = 0.0
C
      IF ( KEY5 ) 190, 140, 160
C
C --- KEY5 = 0 : NO GENERATION OF SYNTHETIC ACCELERATION TIME HISTORY
C
  140 WRITE (6, 1600)
      NIT1 = 0
      NIT = 0
      GO TO 340
C
C --- KEY5 > 0 : GENERATION OF SYNTHETIC ACCELERATION TIME HISTORY BASED
C                ON A BUILT-IN ACCELERATION TIME HISTORY
C
  160 WRITE (6, 1620) KEY5
C
C --- CHOOSES A BUILT-IN ACCELERATION TIME HISTORY BASED ON MW AND R
C     RANGES
C
C   NJG
C     PARAMETERS FOR BUILT-IN TIME HISTORIES
C
C
      NHEAD = 10
      FMT   = '(5E15.7)'
      DT    = 0.005
      PFR1  = 0.0244141
      DPFR  = PFR1
      NP2   = 13
      NPT2  = 8194
      NPT   = 8192
      NPFR  = 4097
      NPFR1 = NPFR - 1

C     ***MAGNITUDE SELECTION LOGIC CHANGED (January, 2006 by Nick Gregor)***
      IF ( MW .LT. 4.5 ) THEN
         IF ( R .LE. 30.0 ) THEN
            AFILE = 'M4-NF.DAT'
            NPIA = 3436
            GO TO 170
         ELSE
            AFILE = 'M4-FF.DAT'
            NPIA = 7000
            GO TO 170
         END IF
      END IF
      IF ( MW .GE. 4.5 .AND. MW .LT. 5.5 ) THEN
         IF ( R .LE. 30.0 ) THEN
            AFILE = 'M5-NF.DAT'
            NPIA = 4119
            GO TO 170
         ELSE
            AFILE = 'M5-FF.DAT'
            NPIA = 3625
            GO TO 170
         END IF
      END IF
      IF ( MW .GE. 5.5 .AND. MW .LT. 6.5 ) THEN
         IF ( R .LE. 30.0 ) THEN
            AFILE = 'M6-NF.DAT'
            NPIA = 4000
            GO TO 170
         ELSE
            AFILE = 'M6-FF.DAT'
            NPIA = 7997
            GO TO 170
         END IF
      END IF
      IF ( MW .GE. 6.5 .AND. MW .LT. 7.0 ) THEN
         IF ( R .LE. 30.0 ) THEN
            AFILE = 'M675-NF.DAT'
            NPIA = 5000
            GO TO 170
         ELSE
            AFILE = 'M675-FF.DAT'
            NPIA = 5000
            GO TO 170
         END IF
      END IF
      IF ( MW .GE. 7.0 .AND. MW .LT. 7.5 ) THEN
         IF ( R .LE. 30.0 ) THEN
            AFILE = 'M725-NF.DAT'
            NPIA = 6000
            GO TO 170
         ELSE
            AFILE = 'M725-FF.DAT'
            NPIA = 5000
            GO TO 170
         END IF
      END IF
      IF ( R .LE. 30.0 ) THEN
         AFILE = 'M75-NF.DAT'
         NPIA = 10000
      ELSE
         AFILE = 'M75-FF.DAT'
         NPIA = 10000
      END IF
      PFR1 = 0.012207
      DPFR = PFR1
      NP2 = 14
      NPT2 = 16386
      NPT = 16384
      NPFR = 8193
      NPFR1 = NPFR - 1
C
  170 OPEN(7, FILE=AFILE)
      DO 180 I=1,NHEAD
      READ(7,1100) WORD
  180 WRITE (6, 1680) WORD
      READ(7,FMT) ( A(I), I=1,NPIA )
      CLOSE(7)
      WRITE (6, 1640) AFILE, MW, R, NPIA, DT
      GO TO 280
C
C --- KEY5 < 0 : GENERATION OF SYNTHETIC ACCELERATION TIME HISTORY BASED
C                ON THE INPUT ACCELERATION TIME HISTORY
C
  190 WRITE (6, 1660) KEY5
C
      READ(5,1100) AFILE
      READ(5,1700) NPIA, NHEAD, DT, FMT
      WRITE (6, 1640) AFILE, MW, R, NPIA, DT
C
      IF ( NPIA .GE. MAXT) WRITE (6,1711)NPIA, MAXT
      IF ( NPIA .GT. MAXT ) THEN
         NPIA = MAXT
         WRITE (6, 1710)
      END IF
C
      IF ( FMT .EQ. '        ' ) FMT = '(8F9.6)'
C
      OPEN(7, FILE=AFILE)
      DO 220 I=1,NHEAD
      READ(7,1100) WORD
  220 WRITE (6, 1680) WORD
      READ(7,FMT) ( A(I), I=1,NPIA)
      CLOSE(7)
C
C --- REDEFINE NPT IF NPIA < 2048
C
      NP2 = NINT( ALOG(FLOAT(NPIA)) / ALOG(2.0) + 0.6 )
      NP2TEST = NINT( ALOG(FLOAT(MAXT)) / ALOG(2.0) )
      IF ( NP2 .GT. NP2TEST ) THEN
         WRITE (6, 1720) NP2TEST, NP2
         NP2 = NP2TEST
      END IF
C
      NPT = 2 ** NP2
      NPT2 = NPT + 2
C
      NPFR = NPT / 2 + 1
      PFR1 = 1.0 / ( NPT * DT )
      DPFR = PFR1
C
C --- DETERMINES THE NYQUIST FREQUENCY OF THE INPUT ACCELERATION TIME
C     HISTORY AND THE UPPER LIMIT OF THE RESPONSE SPECTRAL FREQUENCY
C     BY TAKING 70 % OF THE NYQUIST FREQUENCY.
C
      FNYQ = 0.5 / DT
      FNYQ = 0.7 * FNYQ
      IF ( FNYQ .LT. 34.0 ) THEN
         DO 240 K=1,NFREQ
         IF ( FNYQ .LT. FREQ(K) ) GO TO 260
  240    CONTINUE
  260    NFREQ = K - 1
      END IF
C
  280 WRITE (6, 1730) NPFR, PFR1, DPFR
C
C --- READS IN NORMALIZING FACTORS, OR DESIGN (TARGET) RESPONSE SPECTRUM
C
      READ(5,*) FACTA, FACTV, FACTD
      WRITE (6, 1740) FACTA, FACTV, FACTD
C
      READ(5,*) NIT1, NIT2
      NIT = NIT1 + NIT2
      WRITE (6, 1750) NIT, NIT1, NIT2
C
      IF ( NIT .EQ. 0 ) GO TO 340
C
      WRITE (6, 1200) TITLE
      WRITE (6, 1760)
      READ(5,*) NPSV, ( TFREQ(I), TSV(I), I=1,ABS(NPSV) )
C
      IF ( NPSV .GT. 0 ) THEN
         WRITE (6, 1770)
      ELSE
         WRITE (6, 1780)
      END IF
C
      DO 300 I=1,ABS(NPSV)
      IF ( TSVMX .LE. TSV(I) ) TSVMX = TSV(I)
  300 WRITE (6, 1480) I, TFREQ(I), TSV(I)
C
C --- INTERPOLATES DESIGN (TARGET) RESPONSE SPECTRUM, TSV AT THE
C     OSCILLATOR FREQUENCY, FREQ(I)
C
      DO 320 I=1,NFREQ
      CALL INTERP( TFREQ, TSV, ABS(NPSV), FREQ(I), BSV )
      TSVI(I) = BSV
  320 CONTINUE
C
C --- INITIALIZES NUMBER OF ITERATIONS TO SUFFIXES, CIT1 AND CIT
C
C 340 OPEN(21, STATUS='SCRATCH')
  340 OPEN(21)
      WRITE(21,'(2I2)') NIT1, NIT
      REWIND 21
      READ(21,'(4A1)') CIT1, CIT
      IF ( CIT1(1) .EQ. ' ' ) CIT1(1) = '0'
      IF ( CIT(1) .EQ. ' ' ) CIT(1) = '0'
      CLOSE(21)
C
C --- FUNCTIONAL KEY6 OPERATION
C
C     I = INDEX(OFILE, ' ')
      OPEN (21)
      WRITE (21,'(A80)') OFILE
      REWIND 21
      READ (21,'(80A1)') OFILE1
      CLOSE (21)
      DO 350 J=1,80
         IF (OFILE1(J) .EQ. ' ') THEN
            I = J-1
            GO TO 351
         ENDIF
  350 CONTINUE
  351 OPEN (21)
      WRITE (21,'(''('',I2,''A1,A2,2A1)'')') I
      REWIND 21
      READ (21, '(A80)') FMAT
      CLOSE (21)
C
      K = 1
      SUF8 = '.R'
C     FILE8 = OFILE(1:I-1)//SUF8//CIT1
      OPEN (21)
      WRITE (21, FMAT) (OFILE1(J),J=1,I),SUF8,CIT1
      REWIND 21
      READ (21, '(A80)') FILE8
      CLOSE (21)
      OPEN(8, FILE=FILE8, STATUS='NEW')
C
      IF ( KEY5 .EQ. 0 ) THEN
         WRITE (6, 1790) FILE8
         GO TO 400
      END IF
C
      WRITE (6, 1800) KEY6
      WRITE (6, 1820) K, FILE8
C
      K = K + 1
      SUF9 = '.S'
C     FILE9 = OFILE(1:I-1)//SUF9//CIT
      OPEN (21)
      WRITE (21, FMAT) (OFILE1(J),J=1,I),SUF9,CIT
      REWIND 21
      READ (21, '(A80)') FILE9
      CLOSE (21)
      WRITE (6, 1820) K, FILE9
      OPEN(9, FILE=FILE9, STATUS='NEW')
C
      K = K + 1
      SUF11 = '.A'
C     FILE11 = OFILE(1:I-1)//SUF11//CIT
      OPEN (21)
      WRITE (21, FMAT) (OFILE1(J),J=1,I),SUF11,CIT
      REWIND 21
      READ (21, '(A80)') FILE11
      CLOSE (21)
      WRITE (6, 1820) K, FILE11
      OPEN(11, FILE=FILE11, STATUS='NEW')
C
      IF ( KEY6 .EQ. 0 ) GO TO 390
C
      GO TO ( 360, 380, 360 ) KEY6
C
  360 K = K + 1
      SUF12 = '.V'
C     FILE12 = OFILE(1:I-1)//SUF12//CIT
      OPEN (21)
      WRITE (21, FMAT) (OFILE1(J),J=1,I),SUF12,CIT
      REWIND 21
      READ (21, '(A80)') FILE12
      CLOSE (21)
      WRITE (6, 1820) K, FILE12
      OPEN(12, FILE=FILE12, STATUS='NEW')
      IF ( KEY6 .NE. 3 ) GO TO 390
C
  380 K = K + 1
      SUF13 = '.D'
C     FILE13 = OFILE(1:I-1)//SUF13//CIT
      OPEN (21)
      WRITE (21, FMAT) (OFILE1(J),J=1,I),SUF13,CIT
      REWIND 21
      READ (21, '(A80)') FILE13
      CLOSE (21)
      WRITE (6, 1820) K, FILE13
      OPEN(13, FILE=FILE13, STATUS='NEW')
C
  390 READ(5,*) FC1, NF1, FC2, NF2
      WRITE (6, 1830) FC1, NF1, FC2, NF2
C
C
  400 CONTINUE
C
      WRITE (6, 1200) TITLE
      WRITE (6, 1840) SDAMP, FC, SR, R
C
      PIR = PI * R / SV
      C = C / 981.0
C
C --- INITIALIZES RELATED ARRAYS, BASICALLY FOR PEAK ACCELERATION AND
C     VELOCITY COMPUTATIONS
C
      DO 420 I=1,NFREQ
      FAS(I) = 0.0
      RSA(I) = 0.0
      RSV(I) = 0.0
      SPRAT(I) = 1.0
      PER(I) = 0.0
      FLAG(I) = ' '
  420 CONTINUE
C
      DO 440 I=3,NFREQ
      PER(I) = 1.0 / FREQ(I)
      WI(I) = PI2 * FREQ(I)
  440 CONTINUE
C
C --- COMPUTES FOURIER AMPLITUDE, FAS IN THE OSCILLATOR FREQUENCY REGIME
C
      CALL BRUNE( 3, NFREQ, FREQ, FAS )
C
C --- COMPUTES FOURIER AMPLITUDE, SFAS IN THE BRUNE'S OR INTEGRATION
C     FREQUENCY REGIME
C
      NSFR = MAXB
      SFR1 = 0.0001
      DSFR = 0.005
C
      DO 500 I=1,NSFR
      SFREQ(I) = SFR1 + DSFR * FLOAT( I - 1 )
      SWI(I) = PI2 * SFREQ(I)
      SPRATS(I) = 1.0
  500 CONTINUE
C
      CALL BRUNE( 1, NSFR, SFREQ, SFAS )
C
C
C --- COMPUTES RESPONSE SPECTRAL ACCELERATION, RSA AND VELOCITY, RSV
C     BY RANDOM VIBRATION THEORY (RVT) FOR NIT1 NUMBER OF ITERATIONS
C
      WRITE (6, 1860)
C
      DO 860 IT=1, NIT1 + 1
C
C --- UPDATES SFAS(I) WITH NEW SPRATS(I)
C
      DO 620 I=1,NSFR
      SFAS(I) = SFAS(I) * SPRATS(I)
  620 CONTINUE
C
      DO 800 I=1,NFREQ
C
C --- UPDATES FAS(I) WITH NEW SPRAT(I)
C
      FAS(I) = FAS(I) * SPRAT(I)
C
      IF ( I .LE. 2 ) THEN
         FDRVT = DRVT
         GO TO 640
      END IF
C
C ------ COMPUTES FDRVT FACTOR, FREQUENCY DEPENDENT DURATON OF THE
C        SOURCE EXCITATION IN THE RANDOM VIBRATION THEORY
C
      RATIO = FREQ(I) * DRVT
      RATIO3 = RATIO * RATIO * RATIO
      RATIO4 = RATIO3 / ( RATIO3 + 1.0 / 3.0 )
      FDRVT = DRVT + RATIO4 / ( WI(I) * DAMP )
C
C ------ COMPUTES OSCILLATOR TRANSFER FUNCTION, H(FREQ,SFREQ)
C
      FREQ2 = FREQ(I) * FREQ(I)
      FREQ4 = FREQ2 * FREQ2
C
  640 DO 700 J=1,NSFR
      SFAS2 = SFAS(J) * SFAS(J)
      SWI2 = SWI(J) * SWI(J)
C
C --- SETS H2 = 1.0 TO COMPUTE PEAK ACCELERATION, AMAX
C
      IF ( I .EQ. 1 ) THEN
         H2 = 1.0
         GO TO 680
      END IF
C
C --- SETS H2 = 1.0 / SWI2 TO COMPUTE PEAK VELOCITY, VMAX
C
      IF ( I .EQ. 2 ) THEN
         H2 = 1.0 / SWI2
         GO TO 680
      END IF
C
      FACT1 = FREQ2 - SFREQ(J) * SFREQ(J)
      FACT2 = DAMP2 * FREQ(I) * SFREQ(J)
      H2 = FREQ4 / ( FACT1 * FACT1 + FACT2 * FACT2 )
C
  680 FUNX0(J) = SFAS2 * H2
      FUNX2(J) = FUNX0(J) * SWI2
      FUNX4(J) = FUNX2(J) * SWI2
C
  700 CONTINUE
C
C ------ INTEGRATES THE OSCILLATOR TRANSFER FUNCTION BY SIMPSON'S RULE
C
      CALL SIMPS( NSFR-1, DSFR, FUNX0, SM0 )
      SM0 = SM0 * 2.0
      ARVT = SNGL(DSQRT( SM0 / FDRVT ))
*      ARVT = SQRT( SM0 / FDRVT )
C
      CALL SIMPS( NSFR-1, DSFR, FUNX2, SM2 )
      SM2 = SM2 * 2.0
      PFZ = SNGL(DSQRT( SM2 / SM0 ) / PI2)
*      PFZ = SQRT( SM2 / SM0 ) / PI2
      SNZ = 2.0 * PFZ * DRVT
C
      IF ( SNZ .LT. 2.0 ) THEN
         FLAG(I) = '*'
         SNZ = 2.0
      END IF
C
      CALL SIMPS( NSFR-1, DSFR, FUNX4, SM4 )
      SM4 = SM4 * 2.0
      PFE = SNGL(DSQRT( SM4 / SM2 ) / PI2)
*      PFE = SQRT( SM4 / SM2 ) / PI2
      SNE = 2.0 * PFE * DRVT
C
      IF ( SNE .LT. 2.0 ) THEN
         FLAG(I) = '*'
         SNE = 2.0
      END IF
C
C     COMPUTES QFACT, THE RATIO OF AMAX TO ARVT.
C
      IF ( SNE .LT. 20.0 ) THEN
C
         NE = INT(SNE)
         BAND = SNGL(SM2 / DSQRT( SM0 * SM4 ))
*         BAND = SM2 / SQRT( SM0 * SM4 )
         BANDL = 1.0
         SUM = 0.0
         SIGN = 1.0
C
         DO 720 L=1,NE
         BANDL = BANDL * BAND
C
         SUM = SUM + SIGN * BINOM(L,NE) * BANDL / SQRT( FLOAT(L) )
C
         SIGN = SIGN * (-1.0)
  720    CONTINUE
C
         QFACT = SRHPI *  SUM
C
      ELSE
         QFAC = SQRT( 2.0 * ALOG(SNZ) )
         QFACT = QFAC + EULER / QFAC
      END IF
C
      IF ( I .EQ. 1 ) THEN
         PFEA = PFE
         PFZA = PFZ
         SNEA = SNE
         SNZA = SNZ
         ARVTA = ARVT
         AMAX = ARVT * QFACT
         GO TO 800
      END IF
C
      IF ( I .EQ. 2 ) THEN
         PFEV = PFE
         PFZV = PFZ
         SNEV = SNE
         SNZV = SNZ
         ARVTV = ARVT * 981.0
         VMAX = ARVTV * QFACT
         GO TO 800
      END IF
C
      RSA(I) = ARVT * QFACT
      RSV(I) = RSA(I) / WI(I) * 981.0
C
  800 CONTINUE
C
      WRITE (6, 1880) IT-1, AMAX, ARVTA, PFEA, PFZA, VMAX, ARVTV, PFEV,
     &        PFZV
C
      IF ( KEY5 .EQ. 0 .OR. NIT1 .EQ. 0 ) GO TO 860
C
C --- COMPUTES SPECTRAL RATIO IN THE OSCILLATOR FREQUENCY REGIME
C
      DO 820 I=3,NFREQ
C
      IF ( NPSV .GT. 0 ) THEN
         SPRAT(I) = TSVI(I) / RSA(I)
      ELSE
         SPRAT(I) = TSVI(I) / RSV(I)
      END IF
C
  820 CONTINUE
C
C --- INTERPOLATES SPECTRAL RATIO, SPRAT AT THE INTEGRATION FREQUENCY,
C     SFREQ(I)
C
      DO 840 I=1,NSFR
      CALL INTERB( FREQ, SPRAT, NFREQ, SFREQ(I), BSPRAT )
      SPRATS(I) = BSPRAT
  840 CONTINUE
C
  860 CONTINUE
C
C
C --- INITIALIZES MAXIMUM RVT SPECTRAL VALUES
C
      FASMX = 0.0
      RSAMX = 0.0
      RSVMX = 0.0
      SPRMX = 0.0
C
C --- PRINTS AND WRITES RVT SPECTRAL VALUES OF THE LAST ITERATION
C
      WRITE (6, 1900)
      WRITE (6, 1200) TITLE
      WRITE(8,1100) TITLE
      WRITE(8,1910)
      WRITE(8,1920) NFREQ-2, DAMP, NIT1
      WRITE (6, 1940) DAMP*100.0, NIT1
      WRITE (6, 1950)
C
      IF ( NPSV .GT. 0 ) THEN
         WRITE (6, 1960)
      ELSE
         WRITE (6, 1970)
      END IF
C
      NLINE = 10
C
      DO 870 I=3,NFREQ
      WRITE (6, 1980) I-2, FREQ(I), FAS(I), RSA(I), RSV(I), TSVI(I),
     &            SPRAT(I), PER(I), FLAG(I)
      WRITE(8,2000) FREQ(I), FAS(I), RSA(I), RSV(I), TSVI(I), SPRAT(I)
      NLINE = NLINE + 1
C
      IF ( NLINE .GE. 60) THEN
         WRITE (6, 1200) TITLE
         WRITE (6, 1940) DAMP*100.0, NIT1
         WRITE (6, 1950)
C
         IF ( NPSV .GT. 0 ) THEN
            WRITE (6, 1960)
         ELSE
            WRITE (6, 1970)
         END IF
C
         NLINE = 10
      END IF
C
      IF ( FASMX .LE. FAS(I) )   FASMX = FAS(I)
      IF ( RSAMX .LE. RSA(I) )   RSAMX = RSA(I)
      IF ( RSVMX .LE. RSV(I) )   RSVMX = RSV(I)
      IF ( SPRMX .LE. SPRAT(I) ) SPRMX = SPRAT(I)
C
  870 CONTINUE
C
      CLOSE(8)
      WRITE (6, 2020) FASMX, RSAMX, RSVMX, TSVMX, SPRMX
C
      IF ( KEY5 .EQ. 0 ) GO TO 999
C
      IF ( NIT1 .EQ. 0 ) GO TO 890
C
C --- UPDATES SFAS WITH THE LAST SPRATS
C
      DO 880 I=1,NSFR
      SFAS(I) = SFAS(I) * SPRATS(I)
  880 CONTINUE
C
C --- COMPUTES PHASE FREQUENCY, PFREQ(I) AND INTERPOLATES SFAS AT
C     PFREQ(I) FOR INPUTTING TO SUBROUTINE TIME
C
  890 DO 900 I=1,NPFR
      PFREQ(I) = PFR1 + DPFR * FLOAT( I - 1 )
      CALL INTERP( SFREQ, SFAS, NSFR, PFREQ(I), BFAS )
      PFAS(I) = BFAS
  900 CONTINUE
C
C --- EXTRACTS THE PHASE OF THE INPUT ACCELERATION TIME HISTORY
C
      CALL TIME( -1, 0.0, 0, 0.0, 0, CP, SP, OTH, OTHMX )
C
C --- COMPUTES OUTPUT ACCELERATION TIME HISTORY AND RESPONSE SPECTRUM
C     OF A SINGLE-DEGREE-OF-FREEDOM SYSTEM (SDF) FOR NIT2 ITERATIONS
C
      DO 902 I=1,NFREQ
      PAA(I) = 0.0
      PRV(I) = 0.0
  902 CONTINUE
C
      DO 912 IT=1, NIT2+1
C
      FACTOR = 1.0
C
      IF ( IT .LT. NIT2+1 ) THEN
         CALL TIME( 0, 0.0, 0, 0.0, 0, CP, SP, OTH, OTHAMX )
      ELSE
         CALL TIME( 0, FC1, NF1, FC2, NF2, CP, SP, OTH, OTHAMX )
C
         IF ( FACTA .EQ. 0.0 ) GO TO 906
C
         IF ( FACTA .GT. 0.0 ) THEN
            FACTOR = FACTA / OTHAMX
            OTHAMX = FACTA
         ELSE
            FACTOR = AMAX / OTHAMX
            OTHAMX = AMAX
         END IF
      END IF
C
CWRITE(*,*) 'FACTOR - AMAX'
C     WRITE(*,'(2E12.5)') FACTOR, AMAX
      DO 904 I=1,NPT
  904 OTH(I) = OTH(I) * FACTOR
C
  906 CALL SPECT( NPT, DT, OTH, PAA, PRV )
C
      DO 908 I=3,NFREQ
C
      FAS(I) = FAS(I) * SPRAT(I)
C
      IF ( NIT2 .EQ. 0 ) GO TO 908
C
      IF ( NPSV .GT. 0 ) THEN
         SPRAT(I) = TSVI(I) / PAA(I)
      ELSE
         SPRAT(I) = TSVI(I) / PRV(I)
      END IF
C
  908 CONTINUE
C
      IF ( IT .EQ. NIT2+1 ) GO TO 912
C
      DO 910 I=1,NPFR
      CALL INTERB( FREQ, SPRAT, NFREQ, PFREQ(I), BSPRAT )
      PFAS(I) = PFAS(I) * BSPRAT
  910 CONTINUE
C
  912 CONTINUE
C
C --- UPDATES FOURIER AMPLITUDE, FAS IN THE OSCILLATOR FREQUENCY REGIME
C     BY BUTTERWORTH FILTERING.
C
      RE = CMPLX( 1.0, 0.0 )
      HS = RE
C
      DO 914 I=3,NFREQ
C
C --- COMPUTES THE FIRST BUTTERWORTH FILTER.
C
      CALL BWORTH( FREQ(I), FC1, NF1, HS )
      IF ( FC1 .LT. 0.0 ) HS = 1.0 / HS
      RE = HS
C
C --- COMPUTES THE SECOND BUTTERWORTH FILTER.
C
      CALL BWORTH( FREQ(I), FC2, NF2, HS )
      IF ( FC2 .LT. 0.0 ) HS = 1.0 / HS
      RE = RE * HS
C
      FAS(I) = FAS(I) * CABS(RE)
C
  914 CONTINUE
C
C --- INITIALIZES MAXIMUM SDF SPECTRAL VALUES
C
      FASMX = 0.0
      PAAMX = 0.0
      PRVMX = 0.0
      SPRMX = 0.0
C
C --- PRINTS AND WRITES SDF SPECTRAL VALUES OF THE LAST ITERATION
C
      WRITE (6, 1200) TITLE
      WRITE(9,1100) TITLE
      WRITE(9,2030)
      WRITE(9,1920) NFREQ-2, DAMP, NIT
      WRITE (6, 2040) DAMP*100.0, NIT
      WRITE (6, 2050)
C
      IF ( NPSV .GT. 0 ) THEN
         WRITE (6, 1960)
      ELSE
         WRITE (6, 1970)
      END IF
C
      NLINE = 10
      DO 916 I=3,NFREQ
      WRITE (6, 1980) I-2, FREQ(I), FAS(I), PAA(I), PRV(I), TSVI(I),
     &            SPRAT(I), PER(I)
      WRITE(9,2000) FREQ(I), FAS(I), PAA(I), PRV(I), TSVI(I), SPRAT(I)
      NLINE = NLINE + 1
C
      IF ( NLINE .GE. 60) THEN
         WRITE (6, 1200) TITLE
         WRITE (6, 2040) DAMP*100.0, NIT
         WRITE (6, 2050)
C
         IF ( NPSV .GT. 0 ) THEN
            WRITE (6, 1960)
         ELSE
            WRITE (6, 1970)
         END IF
C
         NLINE = 10
      END IF
C
      IF ( FASMX .LE. FAS(I) )   FASMX = FAS(I)
      IF ( PAAMX .LE. PAA(I) )   PAAMX = PAA(I)
      IF ( PRVMX .LE. PRV(I) )   PRVMX = PRV(I)
      IF ( SPRMX .LE. SPRAT(I) ) SPRMX = SPRAT(I)
C
  916 CONTINUE
C
      CLOSE(9)
      WRITE (6, 2020) FASMX, PAAMX, PRVMX, TSVMX, SPRMX
C
C --- WRITES OUTPUT ACCELERATION TIME HISTORY
C
      WRITE (6, 1200) TITLE
      WRITE (6, 2200)
C
      WRITE(11,1100) TITLE
      WRITE(11,3000) NIT
      WRITE (6, 3100) FILE11, NPT, DT, OTHAMX
      WRITE(11,3200) NPT, DT, OTHAMX
      WRITE(11,3400) ( OTH(I), I=1,NPT )
      CLOSE(11)
C
      IF ( KEY6 .EQ. 0 ) GO TO 999
C
      FACTA = FACTOR
C
      DO 918 I=1,NPFR
      PFAS(I) = PFAS(I) * 981.0
  918 CONTINUE
C
      GO TO ( 920, 930, 920 ) KEY6
C
C --- COMPUTES OUTPUT VELOCITY TIME HISTORY
C
  920 CALL TIME( 1, FC1, NF1, FC2, NF2, CP, SP, OTH, OTHVMX )
C
      WRITE(12,1100) TITLE
      WRITE(12,4000) NIT
C
      IF ( FACTV .EQ. 0.0 ) GO TO 924
C
      IF ( FACTV .GT. 0.0 ) THEN
         FACTOR = FACTV / OTHVMX
         OTHVMX = FACTV
      ELSE
         FACTOR = VMAX / OTHVMX
         OTHVMX = VMAX
      END IF
C
      DO 922 I=1,NPT
  922 OTH(I) = OTH(I) * FACTOR
C
  924 WRITE (6, 4100) FILE12, NPT, DT, OTHVMX
      WRITE(12,4200) NPT, DT, OTHVMX
      WRITE(12,3400) ( OTH(I), I=1,NPT )
      CLOSE(12)
C
C --- COMPUTES OUTPUT DISPLACEMENT TIME HISTORY
C
  930 CALL TIME( 2, FC1, NF1, FC2, NF2, CP, SP, OTH, OTHDMX )
C
      WRITE(13,1100) TITLE
      WRITE(13,5000) NIT
C
      IF ( FACTD .EQ. 0.0 ) GO TO 934
C
      IF ( FACTD .GT. 0.0 ) THEN
         FACTOR = FACTD / OTHDMX
         OTHDMX = FACTD
      ELSE
         FACTOR = AMAX / OTHAMX
         OTHDMX = OTHDMX * FACTOR
      END IF
C
      DO 932 I=1,NPT
  932 OTH(I) = OTH(I) * FACTOR
C
  934 WRITE (6, 5100) FILE13, NPT, DT, OTHDMX
      WRITE(13,5200) NPT, DT, OTHDMX
      WRITE(13,3400) ( OTH(I), I=1,NPT )
      CLOSE(13)
C
      IF ( KEY6 .EQ. 2 ) GO TO 999
C
      VOA = OTHVMX / OTHAMX
      WRITE (6, 6000) VOA
C
      IF ( KEY6 .EQ. 1 ) GO TO 999
C
      ADOVV = OTHAMX * OTHDMX * 981.0 / ( OTHVMX * OTHVMX )
      WRITE (6, 6200) ADOVV
C
  999 WRITE (6, 9990)
C
      STOP
C
C
C --- FORMAT STATEMENTS
C
 1100 FORMAT(A80)
 1200 FORMAT('1',//5X,'*** ',A80)
 1300 FORMAT(//5X,'*** COMMON OUTPUT FILE NAME : ',A80)
 1340 FORMAT(//5X,'*** INPUT PARAMETERS ***',
     &       //9X,'*** SOURCE INFORMATION ***',
     &       //5X,'STRESS DROP ...................... ',F10.3,' BARS',
     &       //5X,'MEDIUM DENSITY ................... ',F10.3,' GM/CC',
     &       //5X,'EPICENTRAL DISTANCE .............. ',F10.3,' KM',
     &       //5X,'SOURCE DEPTH ..................... ',F10.3,' KM',
     &       //5X,'SHEAR WAVE VELOCITY .............. ',F10.3,' KM/SEC',
     &       //5X,'QUALITY FACTOR, Q ................ ',F10.3,
     &       //5X,'CONTROL FREQUENCY FOR Q FACTOR.... ',F10.3,' HZ',
     &       //5X,'EXPONENTIAL CONSTANT FOR Q ....... ',F10.3,
     &       //5X,'MOMENT MAGNITUDE ................. ',F10.3,
     &       //5X,'SPECTRAL DAMPING ................. ',F10.3,' %',
     &       //5X,'FMAX CORNER FREQUENCY ............ ',F10.3,' HZ',
     &       //5X,'FMAX ORDER NUMBER ................ ',I10,
     &       //5X,'EXPONENTIAL FILTERING CONSTANT.... ',F10.3)
 1380 FORMAT( /9X,'*** FUNCTION KEY INFORMATION ***',
     &       //5X,'KEY1 ..............................',I10,
     &       //5X,'KEY2 ..............................',I10,
     &       //5X,'KEY3 ..............................',I10,
     &       //5X,'KEY4 ..............................',I10,
     &       //5X,'KEY5 ..............................',I10,
     &       //5X,'KEY6 ..............................',I10)
 1400 FORMAT(//5X,'*** FUNCTIONAL KEY DESCRIPTION ***')
 1410 FORMAT(//5X,'*** KEY1 = 0 : HALF-SPACE RESPONSE IS COMPUTED')
 1420 FORMAT(//5X,'*** KEY1 = 1 : SITE RESPONSE IS COMPUTED')
 1430 FORMAT(//20X,'*** SITE INFORMATION ***',
     &       //24X,'DENSITY',6X,'SV',5X,'DAMPING',5X,'THICK',5X,'DEPTH',
     &        /24X,'(GM/CC)',3X,'(M/SEC)',5X,'(%)',8X,'(M)',7X,'(M)',
     &        /16X,55('-'),//16X,'SOIL ',5F10.3,//16X,'ROCK ',2F10.3)
 1440 FORMAT(//5X,'*** KEY2 = 0 : NO FREQUENCY DEPENDENT AMPLIFICATION',
     &            ' FACTORS ARE APPLIED')
 1460 FORMAT(//5X,'*** KEY2 = 1 : FREQUENCY DEPENDENT AMPLIFICATION',
     &            ' FACTORS ARE APPLIED :',//29X,'FREQ.',6X,'AMP.',
     &            /22X,'NO.',3X,'(HERTZ)',4X,'FACTOR',/22X,23('-'),/)
 1480 FORMAT(/20X,I5,2F10.4)
 1500 FORMAT(//5X,'*** KEY3 = 1 : ATTEN = 0.1 / SQRT(R) = ',F10.6,
     &            '   FOR R > 100 KM.')
 1520 FORMAT(//5X,'*** KEY3 = 0 : ATTEN = 1.0 / R = ',F10.6)
 1540 FORMAT(//5X,'*** KEY4 = 0 : HERRMANN''S TERM, HERR = 0.0')
 1560 FORMAT(//5X,'*** KEY4 = 1 : HERRMANN''S TERM, HERR = 0.05 * R = ',
     &            F10.4)
 1580 FORMAT(//20X,'*** SOURCE EXCITATION DURATION, DRVT = 1.0 / FC',
     &             ' + HERR = ',F10.4,'  SECONDS')
 1600 FORMAT(//5X,'*** KEY5 AND KEY6 = 0 : NO SYNTHETIC ACCELERATION',
     &            ' TIME HISTORY IS GENERATED')
 1620 FORMAT(//5X,'*** KEY5 = ',I2,' : SYNTHETIC ACCELERATION TIME',
     &            ' HISTORY IS GENERATED BASED ON A BUILT-IN',
     &            ' ACCELERATION TIME HISTORY')
 1640 FORMAT(//21X,'*** INPUT ACCELERATION TIME HISTORY FILE : ',A80,
     &       //25X,'FOR     MW = ',F5.3,'  AND   R = ',F10.3,' KM.',
     &       //25X,'WITH  NPIA = ',I5,'  AND  DT = ',F10.3,' SEC.')
 1660 FORMAT(//5X,'*** KEY5 = ',I2,' : SYNTHETIC ACCELERATION TIME',
     &            ' HISTORY IS GENERATED BASED ON THE INPUT',
     &            ' ACCELERATION TIME HISTORY')
 1680 FORMAT(/25X,'*** ',A80)
 1700 FORMAT(2I5,F10.3,A40)
 1710 FORMAT(/21X,'*** WARNING : ONLY 2048 POINTS ARE USED IN THE',
     &            ' TIME DOMAIN COMPUTATION !!!')
 1711 FORMAT (/1X,'*** WARNING ',I5,'POINTS WERE INPUT.  MAXT = ',I5)
 1720 FORMAT(/21X,'*** WARNING : NP2 = 12 IS USED IN THE TIME DOMAIN',
     &            ' COMPUTATION INSTEAD OF ',I2,' !!!')
 1730 FORMAT(/21X,'*** FREQUENCY RANGE USED IN COMPUTING MODULUS :',
     &      //25X,'NF = ',I5,'  ,  F1 = ',E15.7,' HERTZ  AND  ',
     &            'DF = ',E15.7,' HERTZ')
 1740 FORMAT(//21X,'*** NORMALIZING FACTORS FOR OUTPUT TIME HISTORY :',
     &       //25X,'FACTA = ',F10.4,' G  ,  FACTV = ',F10.4,' CM/SEC',
     &             '  ,  FACTD = ',F10.4,' CM')
 1750 FORMAT(//21X,'*** NO. OF ITERATIONS = ',I2,' : WITH  NIT1 = ',
     &             I2,'  &  NIT2 = ',I2)
 1760 FORMAT(//21X,'*** DESIGN (TARGET) RESPONSE SPECTRUM :')
 1770 FORMAT(//29X,'FREQ.',8X,'SA',/22X,'NO.',3X,'(HERTZ)',5X,'(G''S)',
     &        /22X,23('-'),/)
 1780 FORMAT(//29X,'FREQ.',7X,'SV',/22X,'NO.',3X,'(HERTZ)',2X,'(CM/SEC)'
     &       ,/22X,23('-'),/)
 1790 FORMAT(//29X,'*** OUTPUT FILE IS : ',A80)
 1800 FORMAT(//5X,'*** KEY6 = ',I1,' : OUTPUT FILES ARE :')
 1820 FORMAT(/20X,I1,' : ',A80)
 1830 FORMAT(//20X,'*** FILTERING PARAMETERS FOR OUTPUT TIME HISTORY :',
     &       //25X,'FC1 = ',F10.4,' HZ  ,  NF1 = ',I5,
     &       //25X,'FC2 = ',F10.4,' HZ  ,  NF2 = ',I5)
 1840 FORMAT(//5X,'*** COMPUTED OUTPUT PARAMETERS ***',
     &       //5X,'MEDIUM DAMPING, 1/2Q ............. ',F10.3,' %',
     &       //5X,'SOURCE CORNER FREQUENCY .......... ',F10.3,' HZ',
     &       //5X,'SOURCE RADIUS .................... ',F10.3,' KM',
     &       //5X,'SOURCE-TO-SITE DISTANCE .......... ',F10.3,' KM')
 1860 FORMAT(//5X,'*** PEAK VALUES ***',//18X,'PEAK ACCELERATION',
     &            ' INFORMATION',14X,'PEAK VELOCITY INFORMATION',
     &       /10X,2(3X,37('-')),/6X,'ITER.',5X,'AMAX',6X,'ARMS',7X,
     &            'PFE',7X,'PFZ',6X,'VMAX',6X,'VRMS',7X,'PFE',7X,
     &            'PFZ',/7X,'NO.',2(5X,'(G''S)'),2(6X,'(HZ)'),2X,
     &            2(2X,'(CM/SEC)'),4X,'(HZ)',6X,'(HZ)',/6X,84('-'),/)
 1880 FORMAT(5X,I4,1X,2(2F10.4,2F10.3))
 1900 FORMAT(///5X,'!!! NOTE : * = FLAG FOR N VALUE < 2.0 !!!')
 1910 FORMAT('OUTPUT RVT SPECTRAL VALUES : FREQ, FAS, RSA, RSV, TSV,',
     &       ' SPRAT')
 1920 FORMAT(I5,1x,F5.3,5X,'ITERATION NO. ',I2)
 1940 FORMAT(/9X,'*** RVT SPECTRAL VALUES AT SPECTRAL DAMPING = ',F5.2,
     &           ' %  AND  AT ITERATION NO. ',I2)
 1950 FORMAT(/15X,'FREQ.',9X,'FAS',12X,'RSA',12X,'RSV',12X,'TSV',9X,
     &            'SPECTRAL',5X,'PERIOD')
 1960 FORMAT(7X,'NO.',5X,'(HZ)',8X,'(G.SEC)',10X,'(G)',9X,'(CM/SEC)',
     &      10X,'(G)',11X,'RATIO',7X,'(SEC)',/7X,98('-'),/)
 1970 FORMAT(7X,'NO.',5X,'(HZ)',8X,'(G.SEC)',10X,'(G)',9X,'(CM/SEC)',
     &       7X,'(CM/SEC)',9X,'RATIO',7X,'(SEC)',/7X,98('-'),/)
 1980 FORMAT(7X,I3,F10.4,5E15.4,F10.4,5X,A1)
 2000 FORMAT(6E15.7)
 2020 FORMAT(7X,98('-'),/7X,'MAX. VALUES :',5E15.4,//)
 2030 FORMAT('OUTPUT SDF SPECTRAL VALUES : FREQ, FAS, PAA, PRV, TSV,',
     &       ' SPRAT')
 2040 FORMAT(/9X,'*** SDF SPECTRAL VALUES AT SPECTRAL DAMPING = ',F5.2,
     &           ' %  AND  AT ITERATION NO. ',I2)
 2050 FORMAT(/15X,'FREQ.',9X,'FAS',12X,'PAA',12X,'PRV',12X,'TSV',9X,
     &            'SPECTRAL',5X,'PERIOD')
 2200 FORMAT(//5X,'*** OUTPUT TIME HISTORY INFORMATION ***')
 3000 FORMAT('OUTPUT ACCELERATION TIME HISTORY (G''S) : NIT = ',I2)
 3100 FORMAT(//9X,'*** ',A80,//13X,'NPT = ',I5,'  ,  DT = ',F10.4,
     &            ' SEC.  AND  AMAX = ',F10.4,' G')
 3200 FORMAT(I5,5X,'DT = ',F10.4,' SEC  ,  AMAX = ',F10.4,' G')
 3400 FORMAT(6E12.5)
 4000 FORMAT('OUTPUT VELOCITY TIME HISTORY (CM/SEC) : NIT = ',I2)
 4100 FORMAT(//9X,'*** ',A80,//13X,'NPT = ',I5,'  ,  DT = ',F10.4,
     &            ' SEC.  AND  VMAX = ',F10.4,' CM/SEC')
 4200 FORMAT(I5,5X,'DT = ',F10.4,' SEC  ,  VMAX = ',F10.4,' CM/SEC')
 5000 FORMAT('OUTPUT DISPLACEMENT TIME HISTORY (CM) : NIT = ',I2)
 5100 FORMAT(//9X,'*** ',A80,//13X,'NPT = ',I5,'  ,  DT = ',F10.4,
     &            ' SEC.  AND  DMAX = ',F10.4,' CM')
 5200 FORMAT(I5,5X,'DT = ',F10.4,' SEC  ,  DMAX = ',F10.4,' CM')
 6000 FORMAT(//5X,'*** VMAX / AMAX = ',F10.4,' CM/SEC/G')
 6200 FORMAT(//5X,'*** AMAX * DMAX / (VMAX * VMAX) = ',F10.4)
 9990 FORMAT(///5X,'***   E N D   O F   P R O G R A M   ***',////)
C
      END
      FUNCTION BINOM( L, N )
C
C     FUNCTION BINOM COMPUTES BINOMIAL COEFFICIENTS,
C     BINOM = N!/(L!(N-L)!).
C     NOTE : L MUST BE .LE. N
C
C
      BINOM = 1.0
C
      DO 10 I=2,N
      IF ( I .LE. L ) BINOM = BINOM / REAL(I)
      IF ( I .LE. (N-L) ) BINOM = BINOM / REAL(I)
      BINOM = BINOM * REAL(I)
   10 CONTINUE
      RETURN
      END
***************************************************************
C$NOFLOATCALLS
C$NODEBUG
      SUBROUTINE BRUNE( NFB, NFE, FREQ, FA )
C
C     SUBROUTINE BRUNE COMPUTES FOURIER AMPLITUDE, FA (G-SEC) BASED ON
C     THE METHOD SUGGESTED BY J. N. BRUNE (BSSA, COL. 75, SEPT. 1970).
C     RELATED PARAMETERS ARE CARRIED OVER FROM THE NAMED COMMON BLOCK,
C     FADATA FROM THE MAIN PROGRAM.
C
C
C$LARGE: FREQ,FA
      INCLUDE 'RASCAL.H'
      DIMENSION  FREQ(1), FA(1)
C
      COMMON  /FADATA/ KEY1, KEY2, C, FC, PIR, Q, FQ, ALPHA, FMAX, N2,
     &                 CAP, PICAP
C
      COMMON  /PIDATA/ PI, PI2
C
      COMMON  /AMDATA/ AFREQ(MAXF), AMFAC(MAXF), NAMP
C
      COMMON  /SMDATA/ SV1, DAMP1, THICK, ARATIO, RD, RK, CK
C
C
C --- SETS SMNUM, A MECHINE DEPENDENT EXPONENT OF THE SMALLEST REAL
C     NUMBER
C
*     SMNUM = -30.0
* SET SMNUM = -36.0 FOR SINGLE PRECISION ON MICROSOFT COMPILER (VERSION 3.2)
      SMNUM = -36.0
      IFLAG = 0
C
      AMPF = 1.0
C
      DO 400 I=NFB,NFE
C
      RATIO = FC / FREQ(I)
C
C --- COMPUTES FREQUENCY DEPENDENT Q FUNCTION
C
      IF ( FQ .EQ. 0.0 .OR. ALPHA .EQ. 0.0 ) THEN
         QFUNCT = Q
      ELSE
         QFUNCT = Q * ( FREQ(I) / FQ ) ** ALPHA
      END IF
C
      ERATIO = PIR * FREQ(I) / QFUNCT
      FRATIO = 1.0 / ( 1.0 + RATIO * RATIO )
C
C --- INTERPOLATES FREQUENCY DEPENDENT AMPLIFICATION FACTOR, AMFAC AT
C     FREQUENCY, FREQ(I)
C
      IF ( KEY2 .EQ. 1 )  CALL INTERP(AFREQ, AMFAC, NAMP, FREQ(I), AMPF)
C
      FA(I) = C * EXP(-ERATIO) * FRATIO * AMPF
C
      IF ( FMAX .EQ. 0.0 .OR. N2 .EQ. 0 ) GO TO 200
C
      IF ( IFLAG .EQ. 1 ) GO TO 100
C
C --- CHECKS IF ( ( FREQ(I) / FMAX ) ** N2 ) < ( 10 ** SMNUM )
C
      CHECK = N2 * ALOG10( FREQ(I) / FMAX )
C
      IF ( CHECK .LT. SMNUM ) THEN
         IFLAG = 0
         GO TO 200
      ELSE
         IFLAG = 1
      END IF
C
C --- COMPUTES BUTTERWORTH FILTER
C
  100 FMFACT = SQRT( 1.0 + ( FREQ(I) / FMAX ) ** N2 )
      FA(I) = FA(I) / FMFACT
C
  200 IF ( CAP .EQ. 0.0 ) GO TO 300
C
C --- COMPUTES NEAR-SITE EXPONENTIAL FILTER
C
      FA(I) = FA(I) * EXP( -PICAP * FREQ(I) )
C
  300 IF ( KEY1 .EQ. 0 ) GO TO 400
C
C --- COMPUTES SITE RESPONSE AT RECEIVER DEPTH, FAS IN THE OSCILLATOR
C     REGIME
C
      RK = PI2 * FREQ(I) / SV1
      CK = RK * DAMP1
C
      CALL SAMP( ARDTH )
C
      FA(I) = FA(I) * ARDTH
C
  400 CONTINUE
C
      RETURN
      END
**************************************************
C$NOFLOATCALLS
C$NODEBUG
      SUBROUTINE BWORTH( F, FC, NP, HS )
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C
C     BWORTH CALCULATES THE RESPONSE OF A NP POLE BUTTERWORTH FILTER
C     UP TO AS MANY POLES AS THE ARRAYS S AND T ARE DIMENSIONED
C
C     F  = FREQUENCY(HZ.)
C     FC = THE CORNER FREQUENCY OF THE FILTER
C     NP = THE NUMBER OF POLES, NEGATIVE FCR HIGH PASS
C     HS = COMPLEX RESPONSE OF THE FILTER
C
C
C     THE FORMULA USED -- H(S) = 1/(S-S1)*(S-S2) ... (S-SK)
C
C                   I*PI*(1/2+((2*K-1)/(2*NP)))
C     WHERE SK = EXP                              K = 1,2, ... NP
C
C           S = I*(F/FC)
C
C     REF:  THEORY AND APPLICATION OF DIGITAL SIGNAL PROCESSING
C           RABINE & GOLD, PAGE 227, PRENTICE-HALL 1975
C
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
      COMPLEX S(20), T(20), AS, BK, HS
C
      N=IABS(NP)
C
C
      HS=CMPLX(1.,0.)
      IF ( N .EQ. 0 ) GO TO 40
      IF ( F .EQ. 0. ) GO TO 40
      SS=ABS(F/FC)
      DO 10 K=1,N
      AN = K
      AK = 3.141592654 * (0.5 + (((2.*AN) -1.) / (2.*N)))
      BK = CMPLX(0.0,AK)
   10 S(K) = CEXP(BK)
C
      AS = CMPLX(0.0,SS)
      IF (NP .LT. 0) AS = 1./AS
      T(1) = AS - S(1)
      IF (N .EQ. 1) GO TO 30
      DO 20 I=2,N
   20 T(I) = (AS - S(I)) * T(I-1)
   30 CONTINUE
C
      HS = 1. / T(N)
   40 CONTINUE
C
      RETURN
      END
*********************************************
C$NOFLOATCALLS
C$NODEBUG
      SUBROUTINE CMPMAX( KUG, DT, UG, PR, W, W2, W3, WD, D, Z )
C
C     SUBROUTINE CMPMAX COMPUTES RESPONSE SPECTRUM AT LOW FREQUENCY
C
C
C$LARGE: UG,Z
      DIMENSION  UG(1), Z(1), C(3), X(2,3)
C
      DO 10 I=1,3
      X(1,I)=0.
   10 Z(I)=0.
C
      F1 = 2.*D/(W3*DT)
      F2 = 1./W2
      F3 = D*W
      F4 = 1./WD
      F5 = F3*F4
      F6 = 2.*F3
       E = EXP(-F3*DT)
      G1 = E*SIN(WD*DT)
      G2 = E*COS(WD*DT)
      H1 = WD*G2 - F3*G1
      H2 = WD*G1 + F3*G2
C
      DO 100 K = 1, KUG
      DUG = UG(K+1) - UG(K)
      Z1 = F2*DUG
      Z2 = F2*UG(K)
      Z3 = F1*DUG
      Z4 = Z1/DT
       B = X(1,1) + Z2 -Z3
       A = F4*X(1,2) + F5*B + F4*Z4
      X(2,1) = A*G1 + B*G2 + Z3 - Z2 - Z1
      X(2,2) = A*H1 - B*H2 - Z4
      X(2,3)=-F6*X(2,2)-W2*X(2,1)
C
      DO 80 L=1,3
      C(L)=ABS(X(2,L))
      IF (C(L) .LT. Z(L)) GO TO 80
      Z(L)=C(L)
   80 X(1,L)=X(2,L)
C
  100 CONTINUE
C
      RETURN
      END
********************************************************
C$NOFLOATCALLS
C$NODEBUG
      SUBROUTINE COOL( N, X, SIGNI )
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C        COOL CALCULATES EITHER THE FORWARD OR INVERSE FINITE
C        FOURIER TRANSFORM OF A COMPLEX SERIES.
C
C                 NX
C        F(J) = SUM X(K)*EXP(I*SIGNI*2*PI*(K-1)*(J-1)/NX)  J=1,NX
C                 K=1
C
C        WHERE NX MUST BE AN EXACT POWER OF 2.
C
C        INPUT...
C          N        LOG(NX) TO THE BASE 2
C          X        COMPLEX VECTOR OF DIMENSION .GE. NX
C          SIGNI    .EQ. -1.0 FOR COMPUTATION OF FORWARD TRANSFORM
C                             (FROM TIME DOMAIN TO FREQUENCY DOMAIN)
C                   .EQ. +1.0 FOR COMPUTATION OF INVERSE TRANSFORM
C                             (FROM FREQUENCY DOMAIN TO TIME DOMAIN)
C
C        OUTPUT...
C          X        THE TRANSFORM IS STORED IN THE POSITION OF THE
C                   ORIGINAL SERIES. THE REAL PART CONTAINS THE
C                   COSINE SERIES AND IS SYMMETRIC ABOUT THE POINT
C                   2**(N-1)+1.  THE IMAGINARY PART CONTAINS THE SINE
C                   SERIES AND IS ASYMMETRIC ABOUT THE POINT
C                   2**(N-1)+1.  POINT 1 IS FOR ZERO FREQUENCY AND
C                   POINT 2**(N-1)+1 IS FOR THE NYQUIST FREQUENCY.
C
C        NOTE...
C          THE COMPLEX FACTOR 1/NX = 1/2**N IS NOT APPLIED.
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C$LARGE: X
      COMPLEX X(1), CARG, CEXP, CW, CTEMP, CPI
C
      CPI = CMPLX(0.0,3.1415965358979*SIGNI)
      LX  = 2**N
      J   = 1
C
      DO 30 I=1,LX
      IF (I .GT. J) GO TO 10
      CTEMP = X(J)
      X(J) = X(I)
      X(I) = CTEMP
   10 M = LX / 2
C
   20 IF (J .LE. M) GO TO 30
      J = J - M
      M = M / 2
      IF (M .GE. 1) GO TO 20
C
   30 J = J + M
      L = 1
C
   40 ISTEP = L + L
      DO 50 M=1,L
      CARG = CPI * (M-1) / L
      CW = CEXP(CARG)
C
      DO 50 I=M,LX,ISTEP
      CTEMP = CW * X(I+L)
      X(I+L) = X(I) - CTEMP
   50 X(I) = X(I) + CTEMP
C
      L = ISTEP
      IF (L .LT. LX) GO TO 40
C
      RETURN
      END
************************************************************
C$NOFLOATCALLS
C$NODEBUG
      SUBROUTINE INTERB (X,Y,N,XX,YY)
C
C     SUBROUTINE INTERB LINEARLY INTERPOLATES ANY POINTS BETWEEN TWO
C     SPECIFIED POINTS. IT INTERPOLATES YY AT XX WITHIN N PAIRS OF
C     (X, Y) POINTS. IF XX < X(1), YY IS SET TO EQUAL TO 1.0, AND
C     IF XX > X(N), YY IS SET TO EQUAL TO 1.0.
C
C$LARGE: X,Y
      DIMENSION X(1),Y(1)
      IF (XX .GT. X(1)) GO TO 10
      YY = 1.0
      RETURN
C
   10 IF (XX .LT. X(N)) GO TO 20
      YY = 1.0
      RETURN
C
   20 DO 30 I=2,N
      IF (XX .LT. X(I)) GO TO 40
   30 CONTINUE
   40 YY = Y(I-1)+(Y(I)-Y(I-1))*(XX-X(I-1))/(X(I)-X(I-1))
C
      RETURN
      END
************************************************************
C$NOFLOATCALLS
C$NODEBUG
      SUBROUTINE INTERP (X,Y,N,XX,YY)
C
C     SUBROUTINE INTERP LINEARLY INTERPOLATES ANY POINTS BETWEEN TWO
C     SPECIFIED POINTS. IT INTERPOLATES YY AT XX WITHIN N PAIRS OF
C     (X, Y) POINTS. IF XX < X(1), YY IS SET TO EQUAL TO Y(1), AND
C     IF XX > X(N), YY IS SET TO EQUAL TO Y(N).
C
C$LARGE: X,Y
      DIMENSION X(1),Y(1)
      IF (XX .GT. X(1)) GO TO 10
      YY = Y(1)
      RETURN
C
   10 IF (XX .LT. X(N)) GO TO 20
      YY = Y(N)
      RETURN
C
   20 DO 30 I=2,N
      IF (XX .LT. X(I)) GO TO 40
   30 CONTINUE
   40 YY = Y(I-1)+(Y(I)-Y(I-1))*(XX-X(I-1))/(X(I)-X(I-1))
C
      RETURN
      END
************************************************************
C$NOFLOATCALLS
C$NODEBUG
      SUBROUTINE SAMP( ARDTH )
C
C     SUBROUTINE SAMP COMPUTES SITE AMPLIFICATION AT RECEIVER DEPTH.
C
C
      COMMON  /SMDATA/ SV1, DAMP1, THICK, ARATIO, RD, RK, CK
C
C
      COMPLEX  CARD, CATH, ATH
C
      RKRD = RK * RD
      CKRD = CK * RD
C
      RKTH = RK * THICK
      CKTH = CK * THICK
C
      CARD = CMPLX( RKRD, CKRD )
      CATH = CMPLX( RKTH, CKTH )
C
      ARD = CABS(CCOS(CARD))
      ATH = CCOS(CATH) - CMPLX( 0.0, 1.0 ) * ARATIO * CSIN(CATH)
      ARDTH = ARD / CABS(ATH)
C
      RETURN
      END
************************************************************
C$NOFLOATCALLS
C$NODEBUG
      SUBROUTINE SIMPS( N, DELTA, FUNX, AREA )
C
C     SUBROUTINE SIMPS INTEGRATES INPUT FUNCTION BY SIMPSON'S RULE,
C     NOTE : N MUST BE AN EVEN NO., & FUNX(I) MUST HAVE (N+1) TERMS.
C
      DOUBLE PRECISION FUNX(1),AREA,SUM
C$LARGE: FUNX
*      REAL FUNX(1),AREA,SUM
C
      SUM = FUNX(1)
C
      DO 10 I=2,N,2
   10 SUM = SUM + 4.0 * FUNX(I) + 2.0 * FUNX(I+1)
      SUM = SUM - FUNX(N+1)
C
      AREA = DELTA * SUM / 3.0
C
      RETURN
      END
************************************************************
C$NOFLOATCALLS
C$NODEBUG
      SUBROUTINE SPECT( NPT, DT, A, PAA, PRV )
C
C     SUBROUTINE SPECT COMPUTES A RESPONSE SPECTRUM OF A SINGLE-DEGREE-
C     OF-FREEDOM SYSTEM IN THE TIME DOMAIN BY SOLVING A SET OF LINEAR
C     SIMULTANEOUS EQUATIONS WITH RECURSION RELATIONSHIPS.
C
C
C$LARGE: A,PAA,PRV
      INCLUDE 'RASCAL.H'
      DIMENSION  A(1), PAA(1), PRV(1), Z(3)
C
      COMMON /SPDATA/ DAMP, NFREQ, WI(MAXP), PER(MAXP)
C
C
      KUG = NPT - 1
      TTEST = 10.0 * DT
      YY = SQRT( 1.0 - DAMP * DAMP )
C
      DO 20 I=3,NFREQ
      PR = PER(I)
      W1 = WI(I)
      W2 = W1 * W1
      W3 = W1 * W2
      WD = W1 * YY
C
      IF ( PR .LT. TTEST ) THEN
C
         CALL UCMPMX( KUG, DT, A, PR, W1, W2, W3, WD, DAMP, Z )
C
      ELSE
C
         CALL CMPMAX( KUG, DT, A, PR, W1, W2, W3, WD, DAMP, Z )
C
      END IF
C
      PRV(I) = W1 * Z(1) * 981.0
      PAA(I) = W2 * Z(1)
C
   20 CONTINUE
C
      RETURN
      END
************************************************************
C$NOFLOATCALLS
C$NODEBUG
      SUBROUTINE TIME( IFLAG, FC1, NF1, FC2, NF2, CP, SP, OTH, OTHMX )
C
C     SUBROUTINE TIME COMPUTES THE FFT OF AN INPUT ACCELERATION TIME
C     HISTORY AND EXTRACTS THE PHASE. IT ALSO COMBINES AN INPUT MODULUS
C     (BRUNE FOURIER AMPLITUDE) WITH THE PHASE AND COMPUTES AN OUTPUT
C     TIME HISTORY (ACCELERATION, VELOCITY OR DISPLACEMENT).
C
C     THIS SUBROUTINE CALLS A SUBROUTINE, COOL (AN FFT ALGARITHM) TO DO
C     THE TRANSFORMATION, AND CALLS A SUBROUTINE, BWORTH (A BUTTERWORTH
C     FILTERING ALGARITHM) TO DO THE FILTERING FOR COMPUTING OUTPUT
C     TIME HISTORIES.
C
C
C$LARGE: CP,SP,OTH
      INCLUDE 'RASCAL.H'
      DIMENSION  CP(1), SP(1), OTH(1)
C
      COMMON  /PIDATA/ PI, PI2
C
      COMMON  /TMDATA/ NPIA, NPT, NPT2, NP2, NPFR, DT,
     &                 A(MAXT), PFREQ(MAXF), PFAS(MAXF)
C
C     COMMON/TEST/XX(MAXT)
      COMPLEX  XX(MAXT), RR, HS
C     COMPLEX  XX, RR, HS
C
C
      IF ( IFLAG .GE. 0 ) GO TO 500
C
C --- COMPUTES THE MEAN OF THE INPUT ACCELERATION TIME HISTORY
C
      SUM = 0.0
      DO 120 I=1,NPIA
  120 SUM = SUM + A(I)
      AMEAN = SUM / FLOAT( NPIA )
C
C --- REMOVES THE MEAN FROM THE INPUT ACCELERATION TIME HISTORY
C
      DO 140 I=1,NPIA
  140 A(I) = A(I) - AMEAN
C
C --- ASSUMES 5 % OF NPIA FROM THE INPUT ACCELERATION TIME HISTORY FOR
C     COMPUTING THE COSINE TAPER AT BOTH ENDS OF THE TIME HISTORY
C
      NCT1 = NINT( 0.05 * FLOAT( NPIA ) )
      NCT2 = NPIA - NCT1 + 1
      CTFAC = PI / FLOAT( NCT1 )
C
C --- COMPUTES COSINE TAPER AT THE FIRST AND LAST NCT1 POINTS FOR THE
C     INPUT ACCELERATION TIME HISTORY
C
      DO 200 I=1,NCT1
  200 A(I) = A(I) * ( 0.5 - 0.5 * COS( CTFAC * FLOAT(I) ) )
C
      DO 220 I=NCT2, NPIA
  220 A(I) = A(I) * ( 0.5 - 0.5 * COS( CTFAC * FLOAT(I) ) )
C
C --- ADDS TRAILING ZEROS TO THE CONDITIONED INPUT ACCELERATION TIME
C     HISTORY
C
      DO 300 I=NPIA+1, NPT
  300 A(I) = 0.0
C
C --- ENTERS FFT CALCULATION

C --- TRANSFORMS THE INPUT ACCELERATION TIME HISTORY TO THE FREQUENCY
C     DOMAIN
C
      DO 400 I=1,NPT
  400 XX(I) = CMPLX( A(I), 0.0 )
C
      CALL COOL( NP2, XX, -1.0 )
C
C --- COMPUTES THE COSINE AND SINE OF PHASE
C
      DO 420 I=1,NPFR
      RB = CABS( XX(I) )
      CP(I) = REAL( XX(I) ) / RB
      SP(I) = AIMAG( XX(I) ) / RB
  420 CONTINUE
C
      RETURN
C
C --- INITIALIZES A DUMMY NORMALIZING COMPLEX VARIABLE, HS, AND RR
C ***	INITIALIZES OTH(1)
C --- ENTERS INVERSE FFT CALCULATION
C
  500 RR = CMPLX( 1.0, 0.0 )
      HS = RR
	OTH(1)=0.
C
C --- XX(I) IS THE COMPLEX FOURIER SPECTRUM ARRAY
C --- XX(1) IS ZERO FREQUENCY ELEMENT ( SET TO ZERO, MEAN = 0.0 )
C
      XX(1) = CMPLX( 0.0, 0.0 )
C
C --- ADDS PHASE ( CP, SP ) OF THE CONDITIONED INPUT ACCELERATION TIME
C     HISTORY TO SCALE THE BRUNE MODULUS, PFAS
C
      DO 600 I=2,NPFR
C
      XX(I) = PFAS(I-1) * CMPLX( CP(I), SP(I) )
C
C --- COMPUTES THE FIRST BUTTERWORTH FILTER
C
      CALL BWORTH( PFREQ(I-1), FC1, NF1, HS )
      IF ( FC1 .LT. 0.0 ) HS = 1.0 / HS
      RR = RR * HS
C
C --- COMPUTES THE SECOND BUTTERWORTH FILTER
C
      CALL BWORTH( PFREQ(I-1), FC2, NF2, HS )
      IF ( FC2 .LT. 0.0 ) HS = 1.0 / HS
      RR = RR * HS
C
      XX(I) = XX(I) * RR
C
      RR = CMPLX( 1.0, 0.0 )
      HS = RR
      PWI = PI2 * PFREQ(I-1)
      IF ( IFLAG .GE. 1 ) HS = ( CMPLX( 0.0, -PWI ) ) ** IFLAG
C
C --- DIVIDES THE SCALED SPECTRUM BY COMPLEX OMEGA ** IFALG :
C     IFLAG = 0 : COMPUTES OUTPUT ACCELERATION TIME HISTORY
C     IFLAG = 1 : COMPUTES OUTPUT VELOCITY TIME HISTORY
C     IFLAG = 2 : COMPUTES OUTPUT DISPLACEMENT TIME HISTORY
C
      XX(I) = XX(I) / ( HS * FLOAT(NPT) * DT )
C
C --- NPT * DT IS THE NORMALIZING FACTOR FOR FFT SUBROUTINE COOL.
C     INPUT ACCELERATION TIME HISTORY OF DELTA FUNCTION OF 1/DT
C     YIELDS SPECTRAL DENSITY OF UNITY.
C
      J = NPT2 - I
      XX(J) = CONJG( XX(I) )
C
      HS = CMPLX( 1.0, 0.0 )
C
  600 CONTINUE
C
C --- COMPUTES THE INVERSE FFT
C
      XX(NPFR) = CMPLX( REAL( XX(NPFR) ), 0.0 )
C
      CALL COOL( NP2, XX, 1.0 )
C
      OTHMX = OTH(1)
      DO 700 I=1,NPT
      OTH(I) = REAL( XX(I) )
      IF ( OTHMX .LE. ABS(OTH(I)) ) OTHMX = ABS(OTH(I))
  700 CONTINUE
C
      RETURN
      END
************************************************************
C$NOFLOATCALLS
C$NODEBUG
      SUBROUTINE UCMPMX( KUG, DT, UG, PR, W, W2, W3, WD, D, Z )
C
C     SUBROUTINE UCMPMX COMPUTES RESPONSE SPECTRUM AT HIGH FREQUENCY
C
C
C$LARGE: UG,Z
      DIMENSION  UG(1), Z(1), C(3), X(2,3)
C
      DO 10 I=1,3
      X(1,I)=0.
   10 Z(I)=0.
C
      F2=1./W2
      F3=D*W
      F4=1./WD
      F5=F3*F4
      F6=2.*F3
C
      DO 100 K=1,KUG
      NS=INT(10.*DT/PR-0.01)+1
      DDT=DT/FLOAT(NS)
      F1=2.*D/W3/DDT
      E=EXP(-F3*DDT)
      G1=E*SIN(WD*DDT)
      G2=E*COS(WD*DDT)
      H1=WD*G2-F3*G1
      H2=WD*G1+F3*G2
      DUG=(UG(K+1)-UG(K))/FLOAT(NS)
      G=UG(K)
      Z1=F2*DUG
      Z3=F1*DUG
      Z4=Z1/DDT
C
      DO 100 IS=1,NS
      Z2=F2*G
      B=X(1,1)+Z2-Z3
      A=F4*X(1,2)+F5*B+F4*Z4
      X(2,1)=A*G1+B*G2+Z3-Z2-Z1
      X(2,2)=A*H1-B*H2-Z4
      X(2,3)=-F6*X(2,2)-W2*X(2,1)
C
      DO 80 L=1,3
      C(L)=ABS(X(2,L))
      IF (C(L) .LT. Z(L)) GO TO 80
      Z(L)=C(L)
   80 X(1,L)=X(2,L)
C
      G=G+DUG
C
  100 CONTINUE
C
      RETURN
      END
